﻿Imports System.Collections.Concurrent
Imports System.Threading.Tasks
Imports Examples.Bing.Cloud.Restaurants.BingRoute

Public Class RestaurantLocator
    Implements IRestaurantLocator


    Private Shared restaurants As List(Of Restaurant)

    Public Shared BingApiKey As String = "AuISdif3bCj44Jn7bqCxMRwSOVDJ9gY3IKnsExkrao57c6sCQh_GxYugm87qRYfh"

    Public Sub New()
        If restaurants Is Nothing Then
            restaurants = New List(Of Restaurant)
            restaurants.Add(New Restaurant("My Restaurant", "Curch Street 100", "New York", "10013"))
            restaurants.Add(New Restaurant("Fish Restaurant", "46th Street 52", "New York", "10017"))
            restaurants.Add(New Restaurant("Hamburger Restaurant", "Second Avenue 20", "New York", "10003"))
            restaurants.Add(New Restaurant("Some Restaurant", "Broadway 2350", "New York", "10024 "))
            restaurants.Add(New Restaurant("Good Restaurant", "43 West 24th Street", "New York", "10010"))
            restaurants.Add(New Restaurant("Popular Restaurant", "40 West 40th Street", "New York", "10018"))
            restaurants.Add(New Restaurant("A Restaurant", "509 E. 6th Street", "New York", "10009"))
        End If
    End Sub

    Public Function GetNear(location As BingGeo.Location) As NearResult Implements IRestaurantLocator.GetNear
        Return Nothing
    End Function

    Public Function GetNear(location As BingRoute.Location) As NearResult
        Dim near = New NearResult
        near.Restaurants = New List(Of RestaurantResult)

        Dim results = New ConcurrentBag(Of RestaurantResult)

        ' Build geo request.
        Dim reverseRequest = New BingGeo.ReverseGeocodeRequest
        reverseRequest.Credentials = New BingGeo.Credentials
        reverseRequest.Credentials.Token = BingApiKey
        reverseRequest.Location = New BingGeo.Location
        reverseRequest.Location.Altitude = location.Altitude
        reverseRequest.Location.Latitude = location.Latitude
        reverseRequest.Location.Longitude = location.Longitude

        ' Get the restaurant's location.
        Dim geoClient = New BingGeo.GeocodeServiceClient
        Dim reverseResponse = geoClient.ReverseGeocode(reverseRequest)

        ' Get the address.
        If reverseResponse.Results.Any() Then
            near.Address = reverseResponse.Results.FirstOrDefault().Address.FormattedAddress
        End If

        ' Build geo request.
        ' Get the restaurant's location.
        ' Restaurant position.
        ' Build route request.
        ' Create the request.
        ' Execute the request.
        ' Add the result to the result list.
        Parallel.ForEach(
            restaurants,
            Sub(resto)
                Try
                    Trace.WriteLine(Date.Now & " - Processing: " & resto.Name)
                    Dim geoRequest = New BingGeo.GeocodeRequest
                    geoRequest.Credentials = New BingGeo.Credentials
                    geoRequest.Credentials.Token = BingApiKey
                    geoRequest.Address = New BingGeo.Address
                    geoRequest.Address.AddressLine = resto.Street
                    geoRequest.Address.PostalCode = resto.PostalCode
                    geoRequest.Address.PostalTown = resto.City
                    Dim geoResponse = geoClient.Geocode(geoRequest)
                    If geoResponse.Results.Any() Then
                        Dim restoLocation = geoResponse.Results.FirstOrDefault().Locations.FirstOrDefault()
                        If restoLocation IsNot Nothing Then
                            Dim fromWaypoint = New Waypoint
                            fromWaypoint.Description = "Current Position"
                            fromWaypoint.Location = New Location
                            fromWaypoint.Location.Altitude = location.Altitude
                            fromWaypoint.Location.Latitude = location.Latitude
                            fromWaypoint.Location.Longitude = location.Longitude
                            Dim toWaypoint = New Waypoint
                            toWaypoint.Description = resto.Name
                            toWaypoint.Location = New Location
                            toWaypoint.Location.Altitude = restoLocation.Altitude
                            toWaypoint.Location.Latitude = restoLocation.Latitude
                            toWaypoint.Location.Longitude = restoLocation.Longitude
                            Dim routeRequest = New RouteRequest
                            routeRequest.Credentials = New Credentials
                            routeRequest.Credentials.ApplicationId = BingApiKey
                            routeRequest.Waypoints = New Waypoint(1) {}
                            routeRequest.Waypoints(0) = fromWaypoint
                            routeRequest.Waypoints(1) = toWaypoint
                            routeRequest.Options = New RouteOptions
                            routeRequest.Options.RoutePathType = RoutePathType.Points
                            routeRequest.UserProfile = New UserProfile
                            routeRequest.UserProfile.DistanceUnit = DistanceUnit.Kilometer
                            Dim routeClient = New RouteServiceClient
                            Dim routeResponse = routeClient.CalculateRoute(routeRequest)
                            If routeResponse.Result IsNot Nothing AndAlso
                                routeResponse.Result.Summary IsNot Nothing Then
                                Dim result = New RestaurantResult
                                result.Name = resto.Name
                                result.Distance = routeResponse.Result.Summary.Distance
                                result.TotalSeconds = routeResponse.Result.Summary.TimeInSeconds
                                results.Add(result)
                            End If
                        End If
                    End If
                Catch ex As Exception
                    Trace.WriteLine(ex.Message)
                End Try
            End Sub)

        ' Get the top 3 restaurants.
        Dim i As Integer = 1
        Dim topRestaurants = results.OrderBy(
            Function(o) o.TotalSeconds).Take(3).Select(
            Function(o)
                o.Index = i
                i += 1
                Return o
            End Function)

        ' Done.
        near.Restaurants.AddRange(topRestaurants)
        Return near
    End Function

End Class