﻿Imports System.ComponentModel
Imports System.Device.Location
Imports Examples.Bing.WP7.Restaurants

Public Class RestaurantsViewModel
    Implements INotifyPropertyChanged

    Public Event PropertyChanged As PropertyChangedEventHandler Implements System.ComponentModel.INotifyPropertyChanged.PropertyChanged

    ''' <summary>
    ''' Gets or sets the result.
    ''' </summary>
    ''' <value>The result.</value>
    Public Property Result As NearResult

    ''' <summary>
    ''' Coordinate watcher.
    ''' </summary>
    Private coordinateWatcher As IGeoPositionWatcher(Of GeoCoordinate)

    Public Sub New()
        WaitForCurrentPosition()
    End Sub

    ''' <summary>
    ''' Gets near restaurants once the current position was found.
    ''' </summary>
    ''' <param name="location">The location.</param>
    Private Sub GetNear(ByVal location As Restaurants.Location)
        Dim client = New RestaurantLocatorClient
        AddHandler client.GetNearCompleted, AddressOf OnGetNearComplete
        client.GetNearAsync(location)
    End Sub

    ''' <summary>
    ''' Display near restaurants.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub OnGetNearComplete(ByVal sender As Object, ByVal e As GetNearCompletedEventArgs)
        Me.Result = e.Result
        Me.Change("Result")
    End Sub

    ''' <summary>
    ''' Waits for gps device to load.
    ''' </summary>
    Private Sub WaitForCurrentPosition()
        ' Simulate coordinate.
        Dim events() As GeoCoordinateEventMock = {New GeoCoordinateEventMock With {.Latitude = 39.25, .Longitude = -76.58, .Time = New TimeSpan(0, 0, 5)}}

        ' Start fake coordinate watcher.
        coordinateWatcher = New EventListGeoLocationMock(events)
        AddHandler coordinateWatcher.StatusChanged, AddressOf OnGWatcherStatusChanged
        coordinateWatcher.Start()
    End Sub

    ''' <summary>
    ''' Watcher status changed.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub OnGWatcherStatusChanged(ByVal sender As Object, ByVal e As GeoPositionStatusChangedEventArgs)
        If e.Status = GeoPositionStatus.Ready Then
            coordinateWatcher.Stop()

            Dim location = New Restaurants.Location
            location.Latitude = coordinateWatcher.Position.Location.Latitude
            location.Longitude = coordinateWatcher.Position.Location.Longitude
            GetNear(location)
        End If
    End Sub

    Private Sub Change(ByVal [property] As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs([property]))
    End Sub

End Class