﻿Imports System.ComponentModel
Imports Microsoft.Phone.Controls.Maps.Platform
Imports Utils.WP7.Bing.BingGeo
Imports System.Collections.ObjectModel
Imports Utils.WP7.Bing.BingRoute
Imports Microsoft.Phone.Controls.Maps
Imports System.Device.Location

Public Class RouteViewModel
    Implements INotifyPropertyChanged

#Region "Properties"
    Private toLocation As Location

    Private fromLocation As Location

    Private _from As Address
    Public Property [From] As Address
        Get
            Return _from
        End Get
        Set(ByVal value As Address)
            _from = value
            Change("From")
        End Set
    End Property

    Private _to As Address
    Public Property [To] As Address
        Get
            Return _to
        End Get
        Set(ByVal value As Address)
            _to = value
            Change("To")
        End Set
    End Property

    Private _startPoint As Location
    Public Property StartPoint As Location
        Get
            Return _startPoint
        End Get
        Set(ByVal value As Location)
            _startPoint = value
            Change("StartPoint")
        End Set
    End Property

    Private _endPoint As Location
    Public Property EndPoint As Location
        Get
            Return _endPoint
        End Get
        Set(ByVal value As Location)
            _endPoint = value
            Change("EndPoint")
        End Set
    End Property

    Private _routePoints As ObservableCollection(Of Location)
    Public Property RoutePoints As ObservableCollection(Of Location)
        Get
            Return _routePoints
        End Get
        Set(ByVal value As ObservableCollection(Of Location))
            _routePoints = value
            Change("RoutePoints")
        End Set
    End Property

    Private _itinerary As ObservableCollection(Of ItineraryItem)
    Public Property Itinerary As ObservableCollection(Of ItineraryItem)
        Get
            Return _itinerary
        End Get
        Set(ByVal value As ObservableCollection(Of ItineraryItem))
            _itinerary = value
            Change("Itinerary")
        End Set
    End Property
#End Region

#Region "Events"
    Public Event RouteResolved As EventHandler

    Private Sub RaiseRouteResolved()
        RaiseEvent RouteResolved(Me, EventArgs.Empty)
    End Sub
#End Region

    Public Sub New()
        Me.From = New Address
        Me.From.AddressLine = "A. Vaucampslaan 42"
        Me.From.PostalTown = "Huizingen"
        Me.From.PostalCode = "1654"
        Me.From.CountryRegion = "Belgium"

        Me.To = New Address
        Me._to.AddressLine = "Prins Boudewijnlaan 26"
        Me._to.PostalTown = "Kontich"
        Me._to.PostalCode = "2550"
        Me._to.CountryRegion = "Belgium"
    End Sub

    ''' <summary>
    ''' Find the route between 2 points.
    ''' </summary>
    Public Sub ResolveRoute()
        GetGeoLocation([From], Sub(l) fromLocation = l)
        GetGeoLocation([To], Sub(l) toLocation = l)
    End Sub

    ' FInd the route between 2 positions.
    Public Sub ResolveRouteFromCurrent()
        ' Real watcher.
        'var coordinateWatcher = new GeoCoordinateWatcher(GeoPositionAccuracy.High);
        'coordinateWatcher.StatusChanged += new EventHandler<GeoPositionStatusChangedEventArgs>(OnWatcherStatusChanged);
        'coordinateWatcher.Start();

        ' Mock watcher.
        Dim events() As GeoCoordinateEventMock = {New GeoCoordinateEventMock With {.Latitude = 50, .Longitude = 6, .Time = New TimeSpan(0, 0, 5)}, New GeoCoordinateEventMock With {.Latitude = 50, .Longitude = 7, .Time = New TimeSpan(0, 15, 0)}}

        Dim coordinateWatcher As IGeoPositionWatcher(Of GeoCoordinate) = New EventListGeoLocationMock(events)
        AddHandler coordinateWatcher.StatusChanged, AddressOf OnWatcherStatusChanged
        coordinateWatcher.Start()

        GetGeoLocation([To], Sub(l) toLocation = l)
    End Sub

    Private Sub OnWatcherStatusChanged(ByVal sender As Object, ByVal e As GeoPositionStatusChangedEventArgs)
        Dim coordinateWatcher = TryCast(sender, IGeoPositionWatcher(Of GeoCoordinate))
        If e.Status = GeoPositionStatus.Ready Then
            coordinateWatcher.Stop()

            ' Get position.
            fromLocation = coordinateWatcher.Position.Location
            LocationLoaded()
        End If
    End Sub

    ''' <summary>
    ''' Calculate the route when the location of the start and end destination have been loaded.
    ''' </summary>
    Private Sub LocationLoaded()
        If fromLocation IsNot Nothing AndAlso toLocation IsNot Nothing Then
            Dim fromWaypoint = New Waypoint
            fromWaypoint.Description = "From"
            fromWaypoint.Location = New Location
            fromWaypoint.Location.Altitude = fromLocation.Altitude
            fromWaypoint.Location.Latitude = fromLocation.Latitude
            fromWaypoint.Location.Longitude = fromLocation.Longitude

            Dim toWaypoint = New Waypoint
            toWaypoint.Description = "To"
            toWaypoint.Location = New Location
            toWaypoint.Location.Altitude = toLocation.Altitude
            toWaypoint.Location.Latitude = toLocation.Latitude
            toWaypoint.Location.Longitude = toLocation.Longitude

            ' Create the request.
            Dim routeRequest = New RouteRequest
            routeRequest.Credentials = New Credentials
            routeRequest.Credentials.ApplicationId = App.BingApiKey
            routeRequest.Waypoints = New System.Collections.ObjectModel.ObservableCollection(Of Waypoint)
            routeRequest.Waypoints.Add(fromWaypoint)
            routeRequest.Waypoints.Add(toWaypoint)
            routeRequest.Options = New RouteOptions
            routeRequest.Options.RoutePathType = RoutePathType.Points
            routeRequest.UserProfile = New Utils.WP7.Bing.BingRoute.UserProfile
            routeRequest.UserProfile.DistanceUnit = Utils.WP7.Bing.BingRoute.DistanceUnit.Kilometer

            ' Execute the request.
            Dim routeClient = New RouteServiceClient("BasicHttpBinding_IRouteService")
            AddHandler routeClient.CalculateRouteCompleted, AddressOf OnRouteComplete
            routeClient.CalculateRouteAsync(routeRequest)
        End If
    End Sub

    ''' <summary>
    ''' Route calculated, show in the UI.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub OnRouteComplete(ByVal sender As Object, ByVal e As CalculateRouteCompletedEventArgs)
        If e.Result IsNot Nothing AndAlso e.Result.Result IsNot Nothing AndAlso e.Result.Result.Legs IsNot Nothing And e.Result.Result.Legs.Any() Then
            Dim result = e.Result.Result
            Dim legs = result.Legs.FirstOrDefault()

            StartPoint = legs.ActualStart
            EndPoint = legs.ActualEnd
            RoutePoints = result.RoutePath.Points
            Itinerary = legs.Itinerary

            RaiseRouteResolved()
        End If
    End Sub

    ''' <summary>
    ''' Find the location for an address.
    ''' </summary>
    ''' <param name="address"></param>
    ''' <param name="callBack"></param>
    Private Sub GetGeoLocation(ByVal address As Address, ByVal callBack As Action(Of Utils.WP7.Bing.BingGeo.GeocodeLocation))
        ' Create the request.
        Dim geoRequest = New GeocodeRequest
        geoRequest.Credentials = New Credentials
        geoRequest.Credentials.ApplicationId = App.BingApiKey
        geoRequest.Address = address

        ' Execute the request and display the results.
        Dim geoClient = New GeocodeServiceClient("BasicHttpBinding_IGeocodeService")
        AddHandler geoClient.GeocodeCompleted, Sub(s, e)
                                                   callBack(e.Result.Results.FirstOrDefault().Locations.FirstOrDefault())
                                                   LocationLoaded()
                                               End Sub
        geoClient.GeocodeAsync(geoRequest)
    End Sub

#Region "INotifyPropertyChanged"
    Public Event PropertyChanged As PropertyChangedEventHandler Implements System.ComponentModel.INotifyPropertyChanged.PropertyChanged

    Private Sub Change(ByVal [property] As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs([property]))
    End Sub
#End Region

End Class