﻿Imports System.Device.Location
Imports System.Threading

' Mock Geo location class library to simulate GeoCoordinate information in the Windows
' * Phone developer tools CTP.
' * 
' * Sample library provided as-is without warranty and should not be used in production applications
' * THIS IS A DEBUG TOOL ONLY
' * 
' * Use:
' * 
'     * Dim watcher As IGeoPostitionWatcher(Of GeoCoordinate)
'     * Dim events() As GeoCoordinateEventMock = {
'     *      New GeoCoordinateEventMock With {.Latitude=34.4, .Longitude=11.2, .Time = New TimeSpan(0,0,5)},
'     *      New GeoCoordinateEventMock With {.Latitude=31.4, .Longitude=21.2, .Time = New TimeSpan(0,0,1)}, 
'     *      New GeoCoordinateEventMock With {.Latitude=34.3, .Longitude=28.2, .Time = New TimeSpan(0,0,2)}, 
'     *      New GeoCoordinateEventMock With {.Latitude=32.4, .Longitude=34.2, .Time = New TimeSpan(0,0,3)},
'     *      New GeoCoordinateEventMock With {.Latitude=31.2, .Longitude=37.2, .Time = New TimeSpan(0,0,4)}, 
'     *      New GeoCoordinateEventMock With {.Latitude=33.73, .Longitude=39.2, .Time = New TimeSpan(0,0,5)},
'     *      New GeoCoordinateEventMock With {.Latitude=31.87, .Longitude=41.2, .Time = New TimeSpan(0,0,6)}, 
'     *      New GeoCoordinateEventMock With {.Latitude=11.81, .Longitude=42.2, .Time = New TimeSpan(0,0,7)}
'     *      };
'     *      
'     * watcher = New EventListGeoLocationMock(events)
'     * AddHandler watcher.StatusChanged, AddressOf watcher_StatusChanged
'     * AddHandler watcher.PositionChanged, AddressOf watcher_PositionChanged
'     * watcher.Start()
' * 
' * 

Public MustInherit Class GeoLocationMock
    Implements IGeoPositionWatcher(Of GeoCoordinate)

#Region "Properties"
    Private _current As GeoPosition(Of GeoCoordinate)
    Private _timer As Timer
#End Region

#Region "Constructors"
    Public Sub New()
        _current = New GeoPosition(Of GeoCoordinate)
        Status = GeoPositionStatus.Initializing
        RaiseStatusChanged()
    End Sub
#End Region

#Region "Methods"
    Private Sub RaiseStatusChanged()
        Dim args As New GeoPositionStatusChangedEventArgs(Me.Status)
        RaiseEvent StatusChanged(Me, args)
    End Sub

    Private Sub RaisePositionChanged()
        Dim args As New GeoPositionChangedEventArgs(Of GeoCoordinate)(_current)
        RaiseEvent PositionChanged(Me, args)
    End Sub

    Public Sub OnTimerCallback(ByVal state As Object)
        Try
            If Status = GeoPositionStatus.Initializing Then
                Status = GeoPositionStatus.NoData
                RaiseStatusChanged()
            End If
            StartGetCurrentPosition()
            Dim [next] As TimeSpan = GetNextInterval()
            _timer.Change([next], [next])
        Catch e As Exception
            Throw
        End Try
    End Sub

    Protected Sub UpdateLocation(ByVal longitude As Double, ByVal latitude As Double)
        Dim location As New GeoCoordinate(latitude, longitude)
        If Not location.Equals(_current.Location) Then
            _current = New GeoPosition(Of GeoCoordinate)(DateTimeOffset.Now, location)
            If Status <> GeoPositionStatus.Ready Then
                Status = GeoPositionStatus.Ready
                RaiseStatusChanged()
            End If
            RaisePositionChanged()
        End If
    End Sub

    Protected MustOverride Function GetNextInterval() As TimeSpan
    Protected MustOverride Sub StartGetCurrentPosition()
#End Region

#Region "IGeoPositionWatcher<GeoCoordinate> Members"
    Public ReadOnly Property Permission As GeoPositionPermission
        Get
            Return GeoPositionPermission.Granted
        End Get
    End Property

    Public ReadOnly Property Position As GeoPosition(Of GeoCoordinate) Implements System.Device.Location.IGeoPositionWatcher(Of System.Device.Location.GeoCoordinate).Position
        Get
            Return _current
        End Get
    End Property

    Public Event PositionChanged As EventHandler(Of GeoPositionChangedEventArgs(Of GeoCoordinate)) Implements System.Device.Location.IGeoPositionWatcher(Of System.Device.Location.GeoCoordinate).PositionChanged

    Public Sub Start(ByVal suppressPermissionPrompt As Boolean) Implements System.Device.Location.IGeoPositionWatcher(Of System.Device.Location.GeoCoordinate).Start
        Start()
    End Sub

    Public Sub Start() Implements System.Device.Location.IGeoPositionWatcher(Of System.Device.Location.GeoCoordinate).Start
        Dim span As TimeSpan = GetNextInterval()
        _timer = New Timer(AddressOf OnTimerCallback, Nothing, span, span)
    End Sub

    Private _Status As GeoPositionStatus
    Public Property Status As GeoPositionStatus
        Get
            Return _Status
        End Get
        Protected Set(ByVal value As GeoPositionStatus)
            _Status = value
        End Set
    End Property

    Public Event StatusChanged As EventHandler(Of GeoPositionStatusChangedEventArgs) Implements System.Device.Location.IGeoPositionWatcher(Of System.Device.Location.GeoCoordinate).StatusChanged

    Public Sub [Stop]() Implements System.Device.Location.IGeoPositionWatcher(Of System.Device.Location.GeoCoordinate).Stop
        _timer.Change(Timeout.Infinite, Timeout.Infinite)
        Status = GeoPositionStatus.Disabled
        RaiseStatusChanged()
    End Sub

    Public Function TryStart(ByVal suppressPermissionPrompt As Boolean, ByVal timeout As TimeSpan) As Boolean Implements System.Device.Location.IGeoPositionWatcher(Of System.Device.Location.GeoCoordinate).TryStart
        Start()
        Return True
    End Function

#End Region


    Public ReadOnly Property Status1 As System.Device.Location.GeoPositionStatus Implements System.Device.Location.IGeoPositionWatcher(Of System.Device.Location.GeoCoordinate).Status
        Get
            Return Nothing
        End Get
    End Property
End Class

Public Class GeoCoordinateEventMock
    Public Property Longitude As Double
    Public Property Latitude As Double
    Public Property Time As TimeSpan
End Class

Public Class EventListGeoLocationMock
    Inherits GeoLocationMock
    Private events As List(Of GeoCoordinateEventMock)
    Private currentEventId As Integer

    Public Sub New(ByVal events As IEnumerable(Of GeoCoordinateEventMock))
        Me.events = New List(Of GeoCoordinateEventMock)(events)
        Me.currentEventId = 0
    End Sub

    Private ReadOnly Property Current As GeoCoordinateEventMock
        Get
            Return events(currentEventId Mod events.Count)
        End Get
    End Property

    Protected Overrides Sub StartGetCurrentPosition()
        Me.UpdateLocation(Current.Longitude, Current.Latitude)
        currentEventId += 1
    End Sub

    Protected Overrides Function GetNextInterval() As TimeSpan
        Return Current.Time
    End Function
End Class
